; $Id: play.pro,v 1.2 2001/11/05 14:14:24 aake Exp $
; ----------------------------------------------------------------------
PRO wave_event, ev
;
;  Event handler.  Gets called when something happens, with info on
;  the event in a structure (ev).
;
  COMMON global, x, f, ft, n, rate, nstep,$
                 p1, x1, y1, p2, x2, y2, p3, x3, y3, ftr, fti, eta
  COMMON widgets, base, buttons, quit, smooth, reset, nper, steps, $
                  window, twindow, play, w1, w2, follow, damping

  n2=n*2                                        ; full length
  plotfft = 1                                   ; ok to plot fft
  psym=1                                        ; 1 = +, 2 = *

  CASE ev.id OF                                 ; event choices

  reset: reset_plot                             ; reset
  nper: nstep=ev.value                          ; slider value
  steps: rate=ev.value                          ; slider value
  damping: eta=ev.value                         ; slider value      * DÆMPNING

  quit: begin                                   ; QUIT
    widget_control, base, /destroy              ; delete widgets
    return
  end

  window: begin                                 ; FUNCTION events
    if ev.press eq 1 then follow=1 $            ; start following
    else if ev.release eq 1 then follow=0       ; stop following
    if follow eq 1 then begin                   ; follow?
      wset,w1 & !p=p1 & !x=x1 & !y=y1           ; function window
      oplot,f,psym=-1,color=0                   ; erase
      xy = convert_coord (ev.x, ev.y, $         ; convert pixel xy ..
                          /device, /to_data)    ; .. to data xy
      tmp=min(abs(xy(0)-x), imin)               ; find nearest point
      if imin gt 0 and imin lt n then $         ; test for end points
        f(imin)=xy(1)                           ; change its f
      oplot,f,psym=-1                           ; show it
      wait,0                                    ; flush
    end
    plotfft = ev.release                        ; =0 while dragging
  end

  twindow: begin                                ; TRANSFORM events
    if ev.press eq 1 then follow=1 $            ; start following
    else if ev.release eq 1 then follow=0       ; stop following
    if follow eq 1 then begin                   ; follow?
      wset,w2 & !p=p2 & !x=x2 & !y=y2           ; transform wnd scaling
      xy = convert_coord (ev.x, ev.y, $         ; convert pixel xy ..
                          /device, /to_data)    ; .. to data xy
      tmp=min(abs(xy(0)-x), imin)               ; find nearest point
      imin = (imin > 1) < n                     ; bounds

      oplot,fti(0:n),psym=psym,color=0          ; erase
      fti(imin)=xy(1)                           ; change its fti
      fti(n2-imin)=-xy(1)                       ; neg freq too
      oplot,fti(0:n),psym=psym                  ; show it

      wset,w1 & !p=p1 & !x=x1 & !y=y1           ; function window
      oplot,f,col=0,psy=-1                      ; erase
      ft=complex(0.,fti)                        ; complex ft
      ff=-float(fft(ft,1))                      ; function
      f=ff(0:n)                                 ; half length
      oplot,f,psy=-1                            ; plot
      wait,0                                    ; flush
    end
    plotfft = ev.release                        ; =0 while dragging
  end

  smooth: begin                                 ; SMOOTH
    wset,w1
    oplot,f,psym=-1,color=0                     ; erase
    k=[findgen(n),1+reverse(findgen(n))]        ; k=[0,1,..,n-1,n,n-1,..,1]
    k0=n/2                                      ; low pass cutoff
    ft=ft*exp(-(k/k0)^2)                        ; low pass filter
    ff=float(-fft(ft,1))                        ; inverse transform
    f=ff(0:n)                                   ; half length
    oplot,f,psym=-1                             ; plot new points
    wait,0                                      ; flush
   end

  play: begin                                   ; PLAY
    wset,w1 & !p=p1 & !x=x1 & !y=y1             ; function window
    ff=[0,f(1:n-1),0,-reverse(f(1:n-1))]        ; make it odd
    ;plot,ff(0:n),yrange=[-1,1],xst=1,psy=-1            ; first plot

    periods=1                                   ; number of periods
    ntime=long(nstep*periods)                   ; number of time steps
    k=[findgen(n),1+reverse(findgen(n))]        ; k=[0,1,2,..,n-1,n,n-1,..,2,1]
    omega=2*!pi*periods/ntime*k                 ; phase change per step
    twait=1./rate                               ; waiting time

    ft=-fft(ff,-1)                              ; Fourier transform
    fprv=ff(0:n)                                ; save previous values
    ftprv=ft(0:n)
    t0=systime(1) & t1=t0                       ; start time

    for t=1,ntime do begin                      ; time loop
;     evol=cos(omega*t)                         ; phase change = phi        SLETTET TIL FORDEL FOR DÆMPNING:
      ;eta = 0.1                                ; damping coefficient         * DÆMPNING (uden slider)
      ;evol=cos(omega*t)*exp(-eta*omega*t)      ; phase change = phi          * DÆMPNING (uden slider)
      evol=cos(omega*t)*exp(-0.001*eta*t*omega) ; phase change = phi          * DÆMPNING (med slider)
      ft1=ft*evol                               ; transform at t=now
      ff=-fft(ft1,1)                            ; inverse transform

      wset,w1 & !p=p1 & !x=x1 & !y=y1           ; function window
      oplot,fprv,color=0,psym=-1                ; overplot with black
      oplot,ff(0:n),psym=-1                     ; new plot

      wset,w2 & !p=p2 & !x=x2 & !y=y2           ; transform window
      oplot,imaginary(ftprv),color=0,psym=psym  ; overplot -> erase
      oplot,imaginary(ft1(0:n)),psym=psym       ; new plot

      t2=systime(1)                             ; present time
      wait,(twait-(t2-t1))>0                    ; how long to wait
      t1=t2                                     ; save event time
      fprv=ff(0:n)                              ; save previous values
      ftprv=ft1(0:n)
    end

    f=ff(0:n)                                   ; new function values
  end

  else: print,'else'                            ; never happens (?)

  ENDCASE                                       ; end of CASEs

  if plotfft eq 1 then begin                    ; wait until mouse release

    wset,w1 & !p=p1 & !x=x1 & !y=y1             ; function window
    ff=[0,f(1:n-1),0,-reverse(f(1:n-1))]        ; make it odd
    plot,f,yrange=[-1,1],psym=-1,title='Function'; plot points

    wset,w2 & !p=p2 & !x=x2 & !y=y2             ; transform window
    ft=-fft(ff,-1)                              ; FFT
    range=0.5                                   ; plot range
    fti=imaginary(ft)                           ; imaginary part
    plot,fti(0:n),psym=psym, $                  ; plot imaginary part
      xrange=[0,n],yrange=[-range,range], $     ; range
      title='Fourier Transform (sine part)'
  end
END
; ----------------------------------------------------------------------
PRO reset_plot
;
;  Set initial data and draw initial plots.
;
  COMMON global, x, f, ft, nc, ratec, nstep,$
                 p1, x1, y1, p2, x2, y2, p3, x3, y3, ftr, fti, eta
  COMMON widgets, base, buttons, quit, smooth, reset, nper, steps, $
                 window, twindow, play, w1, w2, follow, damping

  n=nc
  x=findgen(n+1)                                ; equidistant x
  f=replicate(0.,n+1)                           ; constant f

  ff=[0,f(1:n-1),0,-reverse(f(1:n-1))]          ; make it odd
  plot,f,yrange=[-1,1],psym=-1,title='Function' ; plot points
  p1=!p & x1=!x & y1=!y                         ; save plots scaling etc

  wset,w2                                       ; transform window
  ft=-fft(ff,-1)                                ; FFT
  range=0.5                                     ; plot range

  fti=imaginary(ft)                             ; imaginary part
  plot,fti(1:n),psym=1, $                       ; plot imaginary part
    xrange=[0,n],yrange=[-range,range], $      ; range
    title='Fourier Transform (sine part)'
  p2=!p & x2=!x & y2=!y                         ; save scaling

  wset,w1 & !p=p1 & !x=x1 & !y=y1               ; restore function window
  follow = 0                                    ; dont follow mouse yet
END
; ----------------------------------------------------------------------
PRO play,n,rate=rate
;
;  Interatively draw function or transform, plot time evolution 
;  (incl the Fourier transform), and allow smoothing of the
;  function by filtering through a low pass filter.
;
  COMMON global, x, f, ft, nc, ratec, nstep,$
                 p1, x1, y1, p2, x2, y2, p3, x3, y3, ftr, fti, eta
  COMMON widgets, base, buttons, quit, smooth, reset, nper, steps, $
                 window, twindow, play, w1, w2, follow, damping

  if n_elements(   n) eq 0 then    n=16         ; default 16 points
  if n_elements(rate) eq 0 then rate=20         ; default 20 frames per sec

  nc=n & ratec=rate                             ; in common
  nstep = 180                                   ; 2 degrees per step
  eta = 110           ;110 dæmper præcis overtonen med 50%                       ; * DÆMPNING (med slider)

  base    = widget_base (title= 'Wave experimentarium', /column); base widget
  buttons = widget_base (base, /row)            ; button base

  quit   = widget_button (buttons, value='QUIT')        ; add button
  smooth = widget_button (buttons, value='SMOOTH')      ; add button
  play   = widget_button (buttons, value='PLAY')        ; add button
  reset  = widget_button (buttons, value='RESET')       ; add button
  nper   = widget_slider (buttons, $                    ; slider with ..
    min=30, max=360, value=nstep, $                     ; .. values ..
    title='FRAMES PER PERIOD')                          ; .. and title
  steps  = widget_slider (buttons, max= 30, $           ; add slider with
    min=1, value=rate, title='FRAMES PER SECOND')       ; initial value
  damping  = widget_slider (buttons, max= 200, $        ; add slider with         * DÆMPNING (med slider)
    min=0, value=eta, title='DAMPING')                  ; initial value           * DÆMPNING (med slider)

  window = widget_draw (base, xsize=900, ysize=350, $   ; function window
    /button, /motion, colors=8)                         ; mouse events
  twindow= widget_draw (base, xsize=900, ysize=350,  $  ; transform window
    /button, /motion)                                   ; mouse events

  widget_control, base, /realize                ; make all widgets
  widget_control, get_value=w1,  window         ; get window nr
  widget_control, get_value=w2, twindow         ; get window nr

  wset, w1                                      ; select window
  ; bw                                          ; black on white
  background,[80,100,120]                       ; rgb (blue-green) background
  !x.style=1 & !y.style=1 & !p.multi=0          ; exact axes, single frame

  reset_plot
  xmanager, 'wave', base                        ; over to event handler
END
