; $Id: products.pro,v 1.4 2010/05/11 10:52:52 kg Exp $
PRO quit_event, ev
;
;  Event handler.  Gets called when something happens, with info on
;  the event in a structure (ev).

  widget_control, ev.top, /destroy                      ; delete widgets
END
;-----------------------------------------------------------------------
FUNCTION aver, f
  return, total(f)/n_elements(f)
END

;-----------------------------------------------------------------------
PRO doplot
;
;  Plot cosine function
;
  common wavenumbers, k1, k2
  common cosines, c1, c2
  common windows, w1, w2, w3
  common number, n

  dx=2.*!pi/n
  x=findgen(n+1)*dx                                             ; scale

  !p.psym=-6                                                    ; squares
  symsize=0.6                                                   ; small
  yr=[-1.1,1.1]                                                 ; range

  wset, w1                                                      ; select window
  if c1 then begin                                              ; sine or cosine?
    f1=cos(x*k1)                                                ; cosine
    title1='cos('+strtrim(k1,2)+'x)'
  end else begin
    f1=sin(x*k1)                                                ; sine
    title1='sin('+strtrim(k1,2)+'x)'
  end
  plot,x,f1,symsize=symsize,yrange=yr,title=title1              ; plot
  oplot,[x[0],x[n]],[0,0]                                       ; zero-line

  wset, w2
  if c2 then begin
    f2=cos(x*k2)
    title2='cos('+strtrim(k2,2)+'x)'
  end else begin
    f2=sin(x*k2)
    title2='sin('+strtrim(k2,2)+'x)'
  end
  plot,x,f2,symsize=symsize,yrange=yr,title=title2              ; plot
  oplot,[x[0],x[n]],[0,0]                                       ; zero-line

  wset, w3
  f = f1*f2                                                     ; product
  plot,x,f,symsize=symsize,$
    yrange=yr,title=title1+'*'+title2                           ; plot
  oplot,[x[0],x[n]],[0,0]                                       ; zero-line
  oplot,[x[0],x[n]],[1,1]*aver(f[0:n-1]),$                      ; average
    color=thecolor('yellow')                                    ; yellow

  background,[60,80,100]                                        ; blue-green background
END
;-----------------------------------------------------------------------
PRO slider1, ev
;
;  Event handler.  Gets called when something happens, with info on
;  the event in a structure (ev).
  common wavenumbers, k1, k2

  k1=ev.value                                                   ; slider value
  doplot                                                        ; plots
END
;-----------------------------------------------------------------------
PRO slider2, ev
;
;  Event handler.  Gets called when something happens, with info on
;  the event in a structure (ev).
  common wavenumbers, k1, k2

  k2=ev.value                                                   ; slider value
  doplot                                                        ; plots
END
;-----------------------------------------------------------------------
PRO sin1, ev
  common cosines, c1, c2
  c1=0
  doplot                                                        ; plots
END
;-----------------------------------------------------------------------
PRO cos1, ev
  common cosines, c1, c2
  c1=1
  doplot                                                        ; plots
END
;-----------------------------------------------------------------------
PRO sin2, ev
  common cosines, c1, c2
  c2=0
  doplot                                                        ; plots
END
;-----------------------------------------------------------------------
PRO cos2, ev
  common cosines, c1, c2
  c2=1
  doplot                                                        ; plots
END
;-----------------------------------------------------------------------
PRO products, points
;
  common wavenumbers, k1, k2
  common cosines, c1, c2
  common windows, w1, w2, w3
  common number, n

  default, points, 32
  n=points
  xsize=900
  ysize=230

  base = widget_base (title= 'Products', /column)               ; base widget
  rows = widget_base (base, /col)                               ; row of widgets

  row1  = widget_base (rows, /row)                              ; row with plot
  row1a = widget_base (row1, /row, xsize=100)                   ; base for widgets
  void  = widget_slider (row1a, min=0, max=points/2, value=1, $ ; slider
            event_pro = 'slider1', /vertical, ysize=190)
  col1a = widget_base (row1a, /col, /exclusive)                 ; base for buttons
  void = widget_button (col1a, value='sin', event_pro = 'sin1') ; sine button
  void = widget_button (col1a, value='cos', event_pro = 'cos1') ; cosine button
  widget_control, void, /set_button                             ; set it
  draw1= widget_draw (row1, xsize=xsize, ysize=ysize, colors=8) ; draw base

  row2  = widget_base (rows, /row)
  row2a = widget_base (row2, /row, xsize=100)
  void = widget_slider (row2a, min=0, max=points/2, value=1, $
    event_pro = 'slider2', /vertical, ysize=190)
  col2a = widget_base (row2a, /col, /exclusive)
  void = widget_button (col2a, value='sin', event_pro = 'sin2')
  void = widget_button (col2a, value='cos', event_pro = 'cos2')
  widget_control, void, /set_button
  draw2 = widget_draw (row2, xsize=xsize, ysize=ysize, colors=8)

; new
  row4  = widget_base (rows, /row)
  quit   = widget_button (row4, $                    ; quit ...
    event_pro='quit_event', value='QUIT')               ; ... button
;new end

  row3  = widget_base (rows, /row)
  row3a = widget_base (row3, /row, xsize=100)
  draw3 = widget_draw (row3, xsize=xsize, ysize=ysize, colors=8)

  widget_control, base, /realize                                ; make all widgets
  widget_control, get_value=w1,  draw1                          ; get window nr
  widget_control, get_value=w2,  draw2                          ; get window nr
  widget_control, get_value=w3,  draw3                          ; get window nr

  device, decompose=0
  background,[60,80,100]                                        ; blue-green background
  !x.style=1 & !y.style=3                                       ; exact axes
  !p.multi=0                                                    ; single frame

  k1=1 & k2=1 & c1=1 & c2=1                                     ; initial values
  doplot                                                        ; plots

  xmanager, 'alias', base, /no_block                            ; over to event handler
END
